function xbp = bpass(x,dt,flow,fhigh,iplot)
% function xbp = bpass(x,dt,flow,fhigh)
%--------------------------------------------------------------------------
% Matlab function that will band pass filter time series in matrix x
%
% I    x(1:nsamples,1:ntimeseries)= timeseries matrix
% I    dt = time step (s)
% I    flow and fhigh = lower and upper cut-off frequencies, respectively (hz)
% O    xbp = band passed filtered time  series
% H. Lie, Marintek, 09.04.2002
% T. Kristiansen, Marintek, 12.02.2004 : Gaussian for tampering included
%--------------------------------------------------------------------------
nsamples = length(x(:,1));
N=nsamples;
domega = 2*pi/(dt*nsamples);            % Frequency interval (rad/s)
df=domega/(2*pi);
nlow=floor(flow/df);
nhigh=ceil(fhigh/df);
%
% Make frequency array:
Omega = zeros(nsamples,1);

% Create Omega:
if mod(nsamples,2)==0,
   nh = nsamples/2;
   Omega(1:nh+1) = domega*(0:nh);
   Omega(nh+2:nsamples) = domega*(-nh+1:-1);
else
   nh = (nsamples-1)/2;
   Omega(1:nh+1) = domega*(0:nh); 
   Omega(nh+2:nsamples) = domega*(-nh:-1);
end

%The sample indexes, the range between nlow:nhigh is not included
ivec1=[1:nlow];
ivec2=[nhigh:nh];
ivec3=[(nh+1):(N-nhigh+1)];
ivec4=[(N-nlow+1):N];

%Fast-Fourier Transform the signal
Fx = fft(x);

%---- Band-pass filter signals ----
%
% Indices corresponding to frequences to be filtered off:
%
%Ioff = find(abs(Omega)<flow*2*pi | abs(Omega)>fhigh*2*pi);

%Z = zeros(size(Ioff,1),size(x,2));       % Block of zeros to be inserted
%Fx(Ioff,:) = Z;                          % Zero out unwanted components

%
%
% Gaussian window:
beta=0.005;
alpha=log(3)/(flow*beta);
gamma=log(3)/(fhigh*beta);

fvec1=ivec1*df;
fvec2=ivec2*df;
fvec3=ivec3*df;
fvec4=ivec4*df;

%Filtering exponential ramp function
window1=exp(-(alpha*(fvec1-flow).^2));
window2=exp(-(gamma*(fvec2-fhigh).^2));
window3=exp(-(gamma*(fvec3-(N-nhigh+1)*df).^2));
window4=exp(-(alpha*(fvec4-(N-nlow+1)*df).^2));

Fx0=Fx;                                 %Fx0 is the signal before filtering

Fx(ivec1)=Fx(ivec1).*window1';
Fx(ivec2)=Fx(ivec2).*window2';
Fx(ivec3)=Fx(ivec3).*window3';
Fx(ivec4)=Fx(ivec4).*window4';

xbp = real(ifft(Fx));                      % Transform back to time domain. 

% Plot if iplot==1:
if (iplot==1)
  figure;
  title('Band-pass Filter');
  clf
  a=1;
  b=2;
  FF=Omega/(2*pi);
  ymax=max(abs(Fx0));
  ymax0=ymax*1.1;
  
  subplot(a,b,1)
  plot(FF,abs(Fx0),'k')
  hold on
  plot(FF(ivec1),window1*ymax,'g')
  plot(FF(ivec2),window2*ymax,'g')
  ivec0=[max(ivec1):min(ivec2)];
  plot(FF(ivec0),ones(length(ivec0),1)*ymax,'g')
  xlim([0 60]);
  xlabel('Frequency (Hz)');
  ylabel('Spectral Density');
  legend('Unfiltered signal','Filter function');
  grid on
  
  subplot(a,b,2)
  plot(FF,abs(Fx0),'k')
  hold on
  plot(FF,abs(Fx),'r:')
  xlim([0 5]);
  xlabel('Frequency (Hz)');
  ylabel('Spectral Density');
  legend('Unfiltered signal','Filtered signal');
  grid on
  
  pause(5.0)
end

                                         

